@@ -1,20 +1,20 @@
-/* $Id: module-gconf.c 1426 2007-02-13 15:35:19Z ossman $ */
+/* $Id: module-gconf.c 2047 2007-11-11 23:01:44Z lennart $ */
 
 /***
   This file is part of PulseAudio.
 
   Copyright 2006 Lennart Poettering
- 
+
   PulseAudio is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published
   by the Free Software Foundation; either version 2 of the License,
   or (at your option) any later version.
- 
+
   PulseAudio is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.
- 
+
   You should have received a copy of the GNU Lesser General Public License
   along with PulseAudio; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
@@ -25,7 +25,6 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
 #include <string.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -35,28 +34,22 @@
 #include <sys/wait.h>
 #include <fcntl.h>
 
-#ifdef HAVE_SYS_PRCTL_H
-#include <sys/prctl.h>
-#endif
-#ifdef HAVE_SYS_RESOURCE_H
-#include <sys/resource.h>
-#endif
-
+#include <pulse/xmalloc.h>
 #include <pulsecore/module.h>
 #include <pulsecore/core.h>
 #include <pulsecore/llist.h>
 #include <pulsecore/core-util.h>
 #include <pulsecore/log.h>
 #include <pulse/mainloop-api.h>
-#include <pulse/xmalloc.h>
 #include <pulsecore/core-error.h>
+#include <pulsecore/start-child.h>
 
 #include "module-gconf-symdef.h"
 
-PA_MODULE_AUTHOR("Lennart Poettering")
-PA_MODULE_DESCRIPTION("GConf Adapter")
-PA_MODULE_VERSION(PACKAGE_VERSION)
-PA_MODULE_USAGE("")
+PA_MODULE_AUTHOR("Lennart Poettering");
+PA_MODULE_DESCRIPTION("GConf Adapter");
+PA_MODULE_VERSION(PACKAGE_VERSION);
+PA_MODULE_LOAD_ONCE(TRUE);
 
 #define MAX_MODULES 10
 #define BUF_MAX 2048
@@ -80,7 +73,7 @@ struct module_info {
 struct userdata {
     pa_core *core;
     pa_module *module;
-    
+
     pa_hashmap *module_infos;
 
     pid_t pid;
@@ -95,7 +88,7 @@ struct userdata {
 
 static int fill_buf(struct userdata *u) {
     ssize_t r;
-    assert(u);
+    pa_assert(u);
 
     if (u->buf_fill >= BUF_MAX) {
         pa_log("read buffer overflow");
@@ -111,25 +104,25 @@ static int fill_buf(struct userdata *u) {
 
 static int read_byte(struct userdata *u) {
     int ret;
-    assert(u);
+    pa_assert(u);
 
     if (u->buf_fill < 1)
         if (fill_buf(u) < 0)
             return -1;
 
     ret = u->buf[0];
-    assert(u->buf_fill > 0);
+    pa_assert(u->buf_fill > 0);
     u->buf_fill--;
     memmove(u->buf, u->buf+1, u->buf_fill);
     return ret;
 }
 
 static char *read_string(struct userdata *u) {
-    assert(u);
+    pa_assert(u);
 
     for (;;) {
         char *e;
-        
+
         if ((e = memchr(u->buf, 0, u->buf_fill))) {
             char *ret = pa_xstrdup(u->buf);
             u->buf_fill -= e - u->buf +1;
@@ -143,13 +136,13 @@ static char *read_string(struct userdata *u) {
 }
 
 static void unload_one_module(struct userdata *u, struct module_info*m, unsigned i) {
-    assert(u);
-    assert(m);
-    assert(i < m->n_items);
+    pa_assert(u);
+    pa_assert(m);
+    pa_assert(i < m->n_items);
 
     if (m->items[i].index == PA_INVALID_INDEX)
         return;
-            
+
     pa_log_debug("Unloading module #%i", m->items[i].index);
     pa_module_unload_by_index(u->core, m->items[i].index);
     m->items[i].index = PA_INVALID_INDEX;
@@ -160,9 +153,9 @@ static void unload_one_module(struct userdata *u, struct module_info*m, unsigned
 
 static void unload_all_modules(struct userdata *u, struct module_info*m) {
     unsigned i;
-    
-    assert(u);
-    assert(m);
+
+    pa_assert(u);
+    pa_assert(m);
 
     for (i = 0; i < m->n_items; i++)
         unload_one_module(u, m, i);
@@ -179,11 +172,11 @@ static void load_module(
         int is_new) {
 
     pa_module *mod;
-    
-    assert(u);
-    assert(m);
-    assert(name);
-    assert(args);
+
+    pa_assert(u);
+    pa_assert(m);
+    pa_assert(name);
+    pa_assert(args);
 
     if (!is_new) {
         if (m->items[i].index != PA_INVALID_INDEX &&
@@ -193,18 +186,18 @@ static void load_module(
 
         unload_one_module(u, m, i);
     }
-    
+
     pa_log_debug("Loading module '%s' with args '%s' due to GConf configuration.", name, args);
 
     m->items[i].name = pa_xstrdup(name);
     m->items[i].args = pa_xstrdup(args);
     m->items[i].index = PA_INVALID_INDEX;
-    
+
     if (!(mod = pa_module_load(u->core, name, args))) {
         pa_log("pa_module_load() failed");
         return;
     }
-    
+
     m->items[i].index = mod->index;
 }
 
@@ -212,8 +205,8 @@ static void module_info_free(void *p, void *userdata) {
     struct module_info *m = p;
     struct userdata *u = userdata;
 
-    assert(m);
-    assert(u);
+    pa_assert(m);
+    pa_assert(u);
 
     unload_all_modules(u, m);
     pa_xfree(m->name);
@@ -225,20 +218,23 @@ static int handle_event(struct userdata *u) {
     int ret = 0;
 
     do {
-        if ((opcode = read_byte(u)) < 0)
+        if ((opcode = read_byte(u)) < 0){
+            if (errno == EINTR || errno == EAGAIN)
+                break;
             goto fail;
-        
+        }
+
         switch (opcode) {
             case '!':
                 /* The helper tool is now initialized */
                 ret = 1;
                 break;
-                
+
             case '+': {
                 char *name;
                 struct module_info *m;
                 unsigned i, j;
-                
+
                 if (!(name = read_string(u)))
                     goto fail;
 
@@ -282,16 +278,16 @@ static int handle_event(struct userdata *u) {
                 /* Unload all removed modules */
                 for (j = i; j < m->n_items; j++)
                     unload_one_module(u, m, j);
-                    
+
                 m->n_items = i;
-                
+
                 break;
             }
-                
+
             case '-': {
                 char *name;
                 struct module_info *m;
-                
+
                 if (!(name = read_string(u)))
                     goto fail;
 
@@ -301,7 +297,7 @@ static int handle_event(struct userdata *u) {
                 }
 
                 pa_xfree(name);
-                
+
                 break;
             }
         }
@@ -324,101 +320,22 @@ static void io_event_cb(
     struct userdata *u = userdata;
 
     if (handle_event(u) < 0) {
-        
+
         if (u->io_event) {
             u->core->mainloop->io_free(u->io_event);
             u->io_event = NULL;
         }
-            
+
         pa_module_unload_request(u->module);
     }
 }
 
-static int start_client(const char *n, pid_t *pid) {
-    pid_t child;
-    int pipe_fds[2] = { -1, -1 };
-
-    if (pipe(pipe_fds) < 0) {
-        pa_log("pipe() failed: %s", pa_cstrerror(errno));
-        goto fail;
-    }
-    
-    if ((child = fork()) == (pid_t) -1) {
-        pa_log("fork() failed: %s", pa_cstrerror(errno));
-        goto fail;
-    } else if (child != 0) {
-
-        /* Parent */
-        close(pipe_fds[1]);
-
-        if (pid)
-            *pid = child;
-
-        return pipe_fds[0];
-    } else {
-        int max_fd, i;
-        
-        /* child */
-
-        close(pipe_fds[0]);
-        dup2(pipe_fds[1], 1);
-
-        if (pipe_fds[1] != 1)
-            close(pipe_fds[1]);
-
-        close(0);
-        open("/dev/null", O_RDONLY);
-
-        close(2);
-        open("/dev/null", O_WRONLY);
-
-        max_fd = 1024;
-        
-#ifdef HAVE_SYS_RESOURCE_H
-        {
-            struct rlimit r;
-            if (getrlimit(RLIMIT_NOFILE, &r) == 0)
-                max_fd = r.rlim_max;
-        }
-#endif
-                
-        for (i = 3; i < max_fd; i++)
-            close(i);
-
-#ifdef PR_SET_PDEATHSIG
-        /* On Linux we can use PR_SET_PDEATHSIG to have the helper
-        process killed when the daemon dies abnormally. On non-Linux
-        machines the client will die as soon as it writes data to
-        stdout again (SIGPIPE) */
-
-        prctl(PR_SET_PDEATHSIG, SIGTERM, 0, 0, 0);
-#endif
-
-#ifdef SIGPIPE
-        /* Make sure that SIGPIPE kills the child process */
-        signal(SIGPIPE, SIG_DFL);
-#endif
-
-        execl(n, n, NULL);
-        _exit(1);
-    }
-    
-fail:
-    if (pipe_fds[0] >= 0)
-        close(pipe_fds[0]);
-
-    if (pipe_fds[1] >= 0)
-        close(pipe_fds[1]);
-    
-    return -1;
-}
-
-int pa__init(pa_core *c, pa_module*m) {
+int pa__init(pa_module*m) {
     struct userdata *u;
     int r;
 
     u = pa_xnew(struct userdata, 1);
-    u->core = c;
+    u->core = m->core;
     u->module = m;
     m->userdata = u;
     u->module_infos = pa_hashmap_new(pa_idxset_string_hash_func, pa_idxset_string_compare_func);
@@ -427,17 +344,17 @@ int pa__init(pa_core *c, pa_module*m) {
     u->fd_type = 0;
     u->io_event = NULL;
     u->buf_fill = 0;
-    
-    if ((u->fd = start_client(PA_GCONF_HELPER, &u->pid)) < 0)
+
+    if ((u->fd = pa_start_child_for_read(PA_GCONF_HELPER, NULL, &u->pid)) < 0)
         goto fail;
-    
-    u->io_event = c->mainloop->io_new(
-            c->mainloop,
+
+    u->io_event = m->core->mainloop->io_new(
+            m->core->mainloop,
             u->fd,
             PA_IO_EVENT_INPUT,
             io_event_cb,
             u);
-    
+
     do {
         if ((r = handle_event(u)) < 0)
             goto fail;
@@ -445,37 +362,36 @@ int pa__init(pa_core *c, pa_module*m) {
         /* Read until the client signalled us that it is ready with
          * initialization */
     } while (r != 1);
-        
+
     return 0;
 
 fail:
-    pa__done(c, m);
+    pa__done(m);
     return -1;
 }
 
-void pa__done(pa_core *c, pa_module*m) {
+void pa__done(pa_module*m) {
     struct userdata *u;
 
-    assert(c);
-    assert(m);
+    pa_assert(m);
 
     if (!(u = m->userdata))
         return;
 
-    if (u->io_event)
-        c->mainloop->io_free(u->io_event);
-
-    if (u->fd >= 0)
-        close(u->fd);
-
     if (u->pid != (pid_t) -1) {
         kill(u->pid, SIGTERM);
         waitpid(u->pid, NULL, 0);
     }
 
+    if (u->io_event)
+        m->core->mainloop->io_free(u->io_event);
+
+    if (u->fd >= 0)
+        pa_close(u->fd);
+
+
     if (u->module_infos)
         pa_hashmap_free(u->module_infos, module_info_free, u);
 
     pa_xfree(u);
 }
-
