@@ -1,4 +1,4 @@
-/* $Id: module-volume-restore.c 1451 2007-05-23 17:12:07Z lennart $ */
+/* $Id: module-volume-restore.c 2067 2007-11-21 01:30:40Z lennart $ */
 
 /***
   This file is part of PulseAudio.
@@ -26,7 +26,6 @@
 #endif
 
 #include <unistd.h>
-#include <assert.h>
 #include <string.h>
 #include <errno.h>
 #include <sys/types.h>
@@ -35,6 +34,8 @@
 #include <ctype.h>
 
 #include <pulse/xmalloc.h>
+#include <pulse/volume.h>
+#include <pulse/timeval.h>
 
 #include <pulsecore/core-error.h>
 #include <pulsecore/module.h>
@@ -44,40 +45,49 @@
 #include <pulsecore/core-subscribe.h>
 #include <pulsecore/sink-input.h>
 #include <pulsecore/source-output.h>
-#include <pulsecore/core-util.h>
 #include <pulsecore/namereg.h>
-#include <pulse/volume.h>
 
 #include "module-volume-restore-symdef.h"
 
-PA_MODULE_AUTHOR("Lennart Poettering")
-PA_MODULE_DESCRIPTION("Automatically restore the volume and the devices of streams")
-PA_MODULE_USAGE("table=<filename>")
-PA_MODULE_VERSION(PACKAGE_VERSION)
+PA_MODULE_AUTHOR("Lennart Poettering");
+PA_MODULE_DESCRIPTION("Automatically restore the volume and the devices of streams");
+PA_MODULE_VERSION(PACKAGE_VERSION);
+PA_MODULE_LOAD_ONCE(TRUE);
+PA_MODULE_USAGE(
+        "table=<filename> "
+        "restore_device=<Restore the device for each stream?> "
+        "restore_volume=<Restore the volume for each stream?>"
+);
 
 #define WHITESPACE "\n\r \t"
-
 #define DEFAULT_VOLUME_TABLE_FILE "volume-restore.table"
+#define SAVE_INTERVAL 10
 
 static const char* const valid_modargs[] = {
     "table",
+    "restore_device",
+    "restore_volume",
     NULL,
 };
 
 struct rule {
     char* name;
-    int volume_is_set;
+    pa_bool_t volume_is_set;
     pa_cvolume volume;
-    char *sink;
-    char *source;
+    char *sink, *source;
 };
 
 struct userdata {
+    pa_core *core;
     pa_hashmap *hashmap;
     pa_subscription *subscription;
-    pa_hook_slot *sink_input_hook_slot, *source_output_hook_slot;
-    int modified;
+    pa_hook_slot
+        *sink_input_new_hook_slot,
+        *sink_input_fixate_hook_slot,
+        *source_output_new_hook_slot;
+    pa_bool_t modified;
     char *table_file;
+    pa_time_event *save_time_event;
 };
 
 static pa_cvolume* parse_volume(const char *s, pa_cvolume *v) {
@@ -85,8 +95,8 @@ static pa_cvolume* parse_volume(const char *s, pa_cvolume *v) {
     long k;
     unsigned i;
 
-    assert(s);
-    assert(v);
+    pa_assert(s);
+    pa_assert(v);
 
     if (!isdigit(*s))
         return NULL;
@@ -143,7 +153,7 @@ static int load_rules(struct userdata *u) {
     while (!feof(f)) {
         struct rule *rule;
         pa_cvolume v;
-        int v_is_set;
+        pa_bool_t v_is_set;
 
         if (!fgets(ln, sizeof(buf_name), f))
             break;
@@ -170,7 +180,7 @@ static int load_rules(struct userdata *u) {
             continue;
         }
 
-        assert(ln == buf_source);
+        pa_assert(ln == buf_source);
 
         if (buf_volume[0]) {
             if (!parse_volume(buf_volume, &v)) {
@@ -178,9 +188,9 @@ static int load_rules(struct userdata *u) {
                 goto finish;
             }
 
-            v_is_set = 1;
+            v_is_set = TRUE;
         } else
-            v_is_set = 0;
+            v_is_set = FALSE;
 
         ln = buf_name;
 
@@ -221,12 +231,17 @@ static int save_rules(struct userdata *u) {
     void *state = NULL;
     struct rule *rule;
 
+    if (!u->modified)
+        return 0;
+
+    pa_log_info("Saving rules...");
+
     f = u->table_file ?
         fopen(u->table_file, "w") :
         pa_open_config_file(NULL, DEFAULT_VOLUME_TABLE_FILE, NULL, &u->table_file, "w");
 
     if (!f) {
-        pa_log("failed to open file '%s': %s", u->table_file, pa_cstrerror(errno));
+        pa_log("Failed to open file '%s': %s", u->table_file, pa_cstrerror(errno));
         goto finish;
     }
 
@@ -250,6 +265,8 @@ static int save_rules(struct userdata *u) {
     }
 
     ret = 0;
+    u->modified = FALSE;
+    pa_log_debug("Successfully saved rules...");
 
 finish:
     if (f) {
@@ -290,6 +307,21 @@ static char* client_name(pa_client *c) {
     return t;
 }
 
+static void save_time_callback(pa_mainloop_api*a, pa_time_event* e, const struct timeval *tv, void *userdata) {
+    struct userdata *u = userdata;
+
+    pa_assert(a);
+    pa_assert(e);
+    pa_assert(tv);
+    pa_assert(u);
+
+    pa_assert(e == u->save_time_event);
+    u->core->mainloop->time_free(u->save_time_event);
+    u->save_time_event = NULL;
+
+    save_rules(u);
+}
+
 static void subscribe_callback(pa_core *c, pa_subscription_event_type_t t, uint32_t idx, void *userdata) {
     struct userdata *u =  userdata;
     pa_sink_input *si = NULL;
@@ -297,8 +329,8 @@ static void subscribe_callback(pa_core *c, pa_subscription_event_type_t t, uint3
     struct rule *r;
     char *name;
 
-    assert(c);
-    assert(u);
+    pa_assert(c);
+    pa_assert(u);
 
     if (t != (PA_SUBSCRIPTION_EVENT_SINK_INPUT|PA_SUBSCRIPTION_EVENT_NEW) &&
         t != (PA_SUBSCRIPTION_EVENT_SINK_INPUT|PA_SUBSCRIPTION_EVENT_CHANGE) &&
@@ -313,7 +345,7 @@ static void subscribe_callback(pa_core *c, pa_subscription_event_type_t t, uint3
         if (!si->client || !(name = client_name(si->client)))
             return;
     } else {
-        assert((t & PA_SUBSCRIPTION_EVENT_FACILITY_MASK) == PA_SUBSCRIPTION_EVENT_SOURCE_OUTPUT);
+        pa_assert((t & PA_SUBSCRIPTION_EVENT_FACILITY_MASK) == PA_SUBSCRIPTION_EVENT_SOURCE_OUTPUT);
 
         if (!(so = pa_idxset_get_by_index(c->source_outputs, idx)))
             return;
@@ -330,24 +362,24 @@ static void subscribe_callback(pa_core *c, pa_subscription_event_type_t t, uint3
             if (!r->volume_is_set || !pa_cvolume_equal(pa_sink_input_get_volume(si), &r->volume)) {
                 pa_log_info("Saving volume for <%s>", r->name);
                 r->volume = *pa_sink_input_get_volume(si);
-                r->volume_is_set = 1;
-                u->modified = 1;
+                r->volume_is_set = TRUE;
+                u->modified = TRUE;
             }
 
             if (!r->sink || strcmp(si->sink->name, r->sink) != 0) {
                 pa_log_info("Saving sink for <%s>", r->name);
                 pa_xfree(r->sink);
                 r->sink = pa_xstrdup(si->sink->name);
-                u->modified = 1;
+                u->modified = TRUE;
             }
         } else {
-            assert(so);
+            pa_assert(so);
 
             if (!r->source || strcmp(so->source->name, r->source) != 0) {
                 pa_log_info("Saving source for <%s>", r->name);
                 pa_xfree(r->source);
                 r->source = pa_xstrdup(so->source->name);
-                u->modified = 1;
+                u->modified = TRUE;
             }
         }
 
@@ -359,26 +391,60 @@ static void subscribe_callback(pa_core *c, pa_subscription_event_type_t t, uint3
 
         if (si) {
             r->volume = *pa_sink_input_get_volume(si);
-            r->volume_is_set = 1;
+            r->volume_is_set = TRUE;
             r->sink = pa_xstrdup(si->sink->name);
             r->source = NULL;
         } else {
-            assert(so);
-            r->volume_is_set = 0;
+            pa_assert(so);
+            r->volume_is_set = FALSE;
             r->sink = NULL;
             r->source = pa_xstrdup(so->source->name);
         }
 
         pa_hashmap_put(u->hashmap, r->name, r);
-        u->modified = 1;
+        u->modified = TRUE;
+    }
+
+    if (u->modified && !u->save_time_event) {
+        struct timeval tv;
+        pa_gettimeofday(&tv);
+        tv.tv_sec += SAVE_INTERVAL;
+        u->save_time_event = u->core->mainloop->time_new(u->core->mainloop, &tv, save_time_callback, u);
     }
 }
 
-static pa_hook_result_t sink_input_hook_callback(pa_core *c, pa_sink_input_new_data *data, struct userdata *u) {
+static pa_hook_result_t sink_input_new_hook_callback(pa_core *c, pa_sink_input_new_data *data, struct userdata *u) {
     struct rule *r;
     char *name;
 
-    assert(data);
+    pa_assert(data);
+
+    /* In the NEW hook we only adjust the device. Adjusting the volume
+     * is left for the FIXATE hook */
+
+    if (!data->client || !(name = client_name(data->client)))
+        return PA_HOOK_OK;
+
+    if ((r = pa_hashmap_get(u->hashmap, name))) {
+        if (!data->sink && r->sink) {
+            if ((data->sink = pa_namereg_get(c, r->sink, PA_NAMEREG_SINK, 1)))
+                pa_log_info("Restoring sink for <%s>", r->name);
+        }
+    }
+
+    pa_xfree(name);
+
+    return PA_HOOK_OK;
+}
+
+static pa_hook_result_t sink_input_fixate_hook_callback(pa_core *c, pa_sink_input_new_data *data, struct userdata *u) {
+    struct rule *r;
+    char *name;
+
+    pa_assert(data);
+
+    /* In the FIXATE hook we only adjust the volum. Adjusting the device
+     * is left for the NEW hook */
 
     if (!data->client || !(name = client_name(data->client)))
         return PA_HOOK_OK;
@@ -389,21 +455,18 @@ static pa_hook_result_t sink_input_hook_callback(pa_core *c, pa_sink_input_new_d
             pa_log_info("Restoring volume for <%s>", r->name);
             pa_sink_input_new_data_set_volume(data, &r->volume);
         }
-
-        if (!data->sink && r->sink) {
-            if ((data->sink = pa_namereg_get(c, r->sink, PA_NAMEREG_SINK, 1)))
-                pa_log_info("Restoring sink for <%s>", r->name);
-        }
     }
 
+    pa_xfree(name);
+
     return PA_HOOK_OK;
 }
 
-static pa_hook_result_t source_output_hook_callback(pa_core *c, pa_source_output_new_data *data, struct userdata *u) {
+static pa_hook_result_t source_output_new_hook_callback(pa_core *c, pa_source_output_new_data *data, struct userdata *u) {
     struct rule *r;
     char *name;
 
-    assert(data);
+    pa_assert(data);
 
     if (!data->client || !(name = client_name(data->client)))
         return PA_HOOK_OK;
@@ -418,12 +481,12 @@ static pa_hook_result_t source_output_hook_callback(pa_core *c, pa_source_output
     return PA_HOOK_OK;
 }
 
-int pa__init(pa_core *c, pa_module*m) {
+int pa__init(pa_module*m) {
     pa_modargs *ma = NULL;
     struct userdata *u;
+    pa_bool_t restore_device = TRUE, restore_volume = TRUE;
 
-    assert(c);
-    assert(m);
+    pa_assert(m);
 
     if (!(ma = pa_modargs_new(m->argument, valid_modargs))) {
         pa_log("Failed to parse module arguments");
@@ -431,27 +494,45 @@ int pa__init(pa_core *c, pa_module*m) {
     }
 
     u = pa_xnew(struct userdata, 1);
+    u->core = m->core;
     u->hashmap = pa_hashmap_new(pa_idxset_string_hash_func, pa_idxset_string_compare_func);
-    u->subscription = NULL;
     u->table_file = pa_xstrdup(pa_modargs_get_value(ma, "table", NULL));
-    u->modified = 0;
-    u->sink_input_hook_slot = u->source_output_hook_slot = NULL;
+    u->modified = FALSE;
+    u->subscription = NULL;
+    u->sink_input_new_hook_slot = u->sink_input_fixate_hook_slot = u->source_output_new_hook_slot = NULL;
+    u->save_time_event = NULL;
 
     m->userdata = u;
 
+    if (pa_modargs_get_value_boolean(ma, "restore_device", &restore_device) < 0 ||
+        pa_modargs_get_value_boolean(ma, "restore_volume", &restore_volume) < 0) {
+        pa_log("restore_volume= and restore_device= expect boolean arguments");
+        goto fail;
+    }
+
+    if (!(restore_device || restore_volume)) {
+        pa_log("Both restrong the volume and restoring the device are disabled. There's no point in using this module at all then, failing.");
+        goto fail;
+    }
+
     if (load_rules(u) < 0)
         goto fail;
 
-    u->subscription = pa_subscription_new(c, PA_SUBSCRIPTION_MASK_SINK_INPUT|PA_SUBSCRIPTION_MASK_SOURCE_OUTPUT, subscribe_callback, u);
-    u->sink_input_hook_slot = pa_hook_connect(&c->hook_sink_input_new, (pa_hook_cb_t) sink_input_hook_callback, u);
-    u->source_output_hook_slot = pa_hook_connect(&c->hook_source_output_new, (pa_hook_cb_t) source_output_hook_callback, u);
+    u->subscription = pa_subscription_new(m->core, PA_SUBSCRIPTION_MASK_SINK_INPUT|PA_SUBSCRIPTION_MASK_SOURCE_OUTPUT, subscribe_callback, u);
+
+    if (restore_device) {
+        u->sink_input_new_hook_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SINK_INPUT_NEW], (pa_hook_cb_t) sink_input_new_hook_callback, u);
+        u->source_output_new_hook_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_NEW], (pa_hook_cb_t) source_output_new_hook_callback, u);
+    }
+
+    if (restore_volume)
+        u->sink_input_fixate_hook_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SINK_INPUT_FIXATE], (pa_hook_cb_t) sink_input_fixate_hook_callback, u);
 
     pa_modargs_free(ma);
     return 0;
 
 fail:
-    pa__done(c, m);
-
+    pa__done(m);
     if (ma)
         pa_modargs_free(ma);
 
@@ -460,7 +541,7 @@ fail:
 
 static void free_func(void *p, void *userdata) {
     struct rule *r = p;
-    assert(r);
+    pa_assert(r);
 
     pa_xfree(r->name);
     pa_xfree(r->sink);
@@ -468,11 +549,10 @@ static void free_func(void *p, void *userdata) {
     pa_xfree(r);
 }
 
-void pa__done(pa_core *c, pa_module*m) {
+void pa__done(pa_module*m) {
     struct userdata* u;
 
-    assert(c);
-    assert(m);
+    pa_assert(m);
 
     if (!(u = m->userdata))
         return;
@@ -480,21 +560,21 @@ void pa__done(pa_core *c, pa_module*m) {
     if (u->subscription)
         pa_subscription_free(u->subscription);
 
-    if (u->sink_input_hook_slot)
-        pa_hook_slot_free(u->sink_input_hook_slot);
-    if (u->source_output_hook_slot)
-        pa_hook_slot_free(u->source_output_hook_slot);
+    if (u->sink_input_new_hook_slot)
+        pa_hook_slot_free(u->sink_input_new_hook_slot);
+    if (u->sink_input_fixate_hook_slot)
+        pa_hook_slot_free(u->sink_input_fixate_hook_slot);
+    if (u->source_output_new_hook_slot)
+        pa_hook_slot_free(u->source_output_new_hook_slot);
 
     if (u->hashmap) {
-
-        if (u->modified)
-            save_rules(u);
-
+        save_rules(u);
         pa_hashmap_free(u->hashmap, free_func, NULL);
     }
 
+    if (u->save_time_event)
+        u->core->mainloop->time_free(u->save_time_event);
+
     pa_xfree(u->table_file);
     pa_xfree(u);
 }
-
-
