@@ -1,20 +1,20 @@
-/* $Id: sdp.c 1426 2007-02-13 15:35:19Z ossman $ */
+/* $Id: sdp.c 1971 2007-10-28 19:13:50Z lennart $ */
 
 /***
   This file is part of PulseAudio.
 
   Copyright 2006 Lennart Poettering
- 
+
   PulseAudio is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published
   by the Free Software Foundation; either version 2 of the License,
   or (at your option) any later version.
- 
+
   PulseAudio is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.
- 
+
   You should have received a copy of the GNU Lesser General Public License
   along with PulseAudio; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
@@ -25,7 +25,6 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
 #include <time.h>
 #include <stdlib.h>
 #include <sys/types.h>
@@ -35,37 +34,34 @@
 #include <string.h>
 
 #include <pulse/xmalloc.h>
+#include <pulse/util.h>
 
 #include <pulsecore/core-util.h>
 #include <pulsecore/log.h>
+#include <pulsecore/macro.h>
 
 #include "sdp.h"
 #include "rtp.h"
 
-
 char *pa_sdp_build(int af, const void *src, const void *dst, const char *name, uint16_t port, uint8_t payload, const pa_sample_spec *ss) {
     uint32_t ntp;
-    char buf_src[64], buf_dst[64];
+    char buf_src[64], buf_dst[64], un[64];
     const char *u, *f, *a;
 
-    assert(src);
-    assert(dst);
-    assert(af == AF_INET || af == AF_INET6);
+    pa_assert(src);
+    pa_assert(dst);
+    pa_assert(af == AF_INET || af == AF_INET6);
+
+    pa_assert_se(f = pa_rtp_format_to_string(ss->format));
+
+    if (!(u = pa_get_user_name(un, sizeof(un))))
+        u = "-";
 
-    f = pa_rtp_format_to_string(ss->format);
-    assert(f);
-    
-    if (!(u = getenv("USER")))
-        if (!(u = getenv("USERNAME")))
-            u = "-";
-    
     ntp = time(NULL) + 2208988800U;
 
-    a = inet_ntop(af, src, buf_src, sizeof(buf_src));
-    assert(a);
-    a = inet_ntop(af, dst, buf_dst, sizeof(buf_dst));
-    assert(a);
-    
+    pa_assert_se(a = inet_ntop(af, src, buf_src, sizeof(buf_src)));
+    pa_assert_se(a = inet_ntop(af, dst, buf_dst, sizeof(buf_dst)));
+
     return pa_sprintf_malloc(
             PA_SDP_HEADER
             "o=%s %lu 0 IN %s %s\n"
@@ -86,8 +82,8 @@ char *pa_sdp_build(int af, const void *src, const void *dst, const char *name, u
 
 static pa_sample_spec *parse_sdp_sample_spec(pa_sample_spec *ss, char *c) {
     unsigned rate, channels;
-    assert(ss);
-    assert(c);
+    pa_assert(ss);
+    pa_assert(c);
 
     if (pa_startswith(c, "L16/")) {
         ss->format = PA_SAMPLE_S16BE;
@@ -123,13 +119,13 @@ pa_sdp_info *pa_sdp_parse(const char *t, pa_sdp_info *i, int is_goodbye) {
     uint16_t port = 0;
     int ss_valid = 0;
 
-    assert(t);
-    assert(i);
-    
+    pa_assert(t);
+    pa_assert(i);
+
     i->origin = i->session_name = NULL;
     i->salen = 0;
     i->payload = 255;
-    
+
     if (!pa_startswith(t, PA_SDP_HEADER)) {
         pa_log("Failed to parse SDP data: invalid header.");
         goto fail;
@@ -156,7 +152,7 @@ pa_sdp_info *pa_sdp_parse(const char *t, pa_sdp_info *i, int is_goodbye) {
             size_t k;
 
             k = l-8 > sizeof(a) ? sizeof(a) : l-8;
-            
+
             pa_strlcpy(a, t+9, k);
             a[strcspn(a, "/")] = 0;
 
@@ -173,7 +169,7 @@ pa_sdp_info *pa_sdp_parse(const char *t, pa_sdp_info *i, int is_goodbye) {
             size_t k;
 
             k = l-8 > sizeof(a) ? sizeof(a) : l-8;
-            
+
             pa_strlcpy(a, t+9, k);
             a[strcspn(a, "/")] = 0;
 
@@ -189,7 +185,7 @@ pa_sdp_info *pa_sdp_parse(const char *t, pa_sdp_info *i, int is_goodbye) {
 
             if (i->payload > 127) {
                 int _port, _payload;
-                
+
                 if (sscanf(t+8, "%i RTP/AVP %i", &_port, &_payload) == 2) {
 
                     if (_port <= 0 || _port > 0xFFFF) {
@@ -224,16 +220,16 @@ pa_sdp_info *pa_sdp_parse(const char *t, pa_sdp_info *i, int is_goodbye) {
                     if (_payload == i->payload) {
 
                         c[strcspn(c, "\n")] = 0;
-                        
+
                         if (parse_sdp_sample_spec(&i->sample_spec, c))
                             ss_valid = 1;
                     }
                 }
             }
         }
-        
+
         t += l;
-        
+
         if (*t == '\n')
             t++;
     }
@@ -247,7 +243,7 @@ pa_sdp_info *pa_sdp_parse(const char *t, pa_sdp_info *i, int is_goodbye) {
         ((struct sockaddr_in*) &i->sa)->sin_port = htons(port);
     else
         ((struct sockaddr_in6*) &i->sa)->sin6_port = htons(port);
-    
+
     return i;
 
 fail:
@@ -258,7 +254,7 @@ fail:
 }
 
 void pa_sdp_info_destroy(pa_sdp_info *i) {
-    assert(i);
+    pa_assert(i);
 
     pa_xfree(i->origin);
     pa_xfree(i->session_name);
