@@ -1,7 +1,7 @@
 #ifndef foosimplehfoo
 #define foosimplehfoo
 
-/* $Id: simple.h 1426 2007-02-13 15:35:19Z ossman $ */
+/* $Id: simple.h 1999 2007-10-30 14:17:41Z lennart $ */
 
 /***
   This file is part of PulseAudio.
@@ -51,7 +51,7 @@
  * pa_simple *s;
  * pa_sample_spec ss;
  *
- * ss.format = PA_SAMPLE_S16_NE;
+ * ss.format = PA_SAMPLE_S16NE;
  * ss.channels = 2;
  * ss.rate = 44100;
  *
@@ -130,13 +130,13 @@ pa_simple* pa_simple_new(
 void pa_simple_free(pa_simple *s);
 
 /** Write some data to the server */
-int pa_simple_write(pa_simple *s, const void*data, size_t length, int *error);
+int pa_simple_write(pa_simple *s, const void*data, size_t bytes, int *error);
 
 /** Wait until all data already written is played by the daemon */
 int pa_simple_drain(pa_simple *s, int *error);
 
 /** Read some data from the server */
-int pa_simple_read(pa_simple *s, void*data, size_t length, int *error);
+int pa_simple_read(pa_simple *s, void*data, size_t bytes, int *error);
 
 /** Return the playback latency. \since 0.5 */
 pa_usec_t pa_simple_get_latency(pa_simple *s, int *error);
