@@ -1,7 +1,7 @@
 #ifndef fooutilhfoo
 #define fooutilhfoo
 
-/* $Id: util.h 1426 2007-02-13 15:35:19Z ossman $ */
+/* $Id: util.h 1971 2007-10-28 19:13:50Z lennart $ */
 
 /***
   This file is part of PulseAudio.
@@ -52,7 +52,7 @@ char *pa_get_binary_name(char *s, size_t l);
 
 /** Return a pointer to the filename inside a path (which is the last
  * component). */
-const char *pa_path_get_filename(const char *p);
+char *pa_path_get_filename(const char *p);
 
 /** Wait t milliseconds */
 int pa_msleep(unsigned long t);
