@@ -1,4 +1,4 @@
-/* $Id: cli.c 1426 2007-02-13 15:35:19Z ossman $ */
+/* $Id: cli.c 2008 2007-11-01 00:32:45Z lennart $ */
 
 /***
   This file is part of PulseAudio.
@@ -27,7 +27,6 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <assert.h>
 #include <stdlib.h>
 
 #include <pulse/xmalloc.h>
@@ -45,6 +44,7 @@
 #include <pulsecore/cli-text.h>
 #include <pulsecore/cli-command.h>
 #include <pulsecore/log.h>
+#include <pulsecore/macro.h>
 
 #include "cli.h"
 
@@ -59,7 +59,8 @@ struct pa_cli {
 
     pa_client *client;
 
-    int fail, kill_requested, defer_kill;
+    pa_bool_t fail, kill_requested;
+    int defer_kill;
 };
 
 static void line_callback(pa_ioline *line, const char *s, void *userdata);
@@ -68,19 +69,17 @@ static void client_kill(pa_client *c);
 pa_cli* pa_cli_new(pa_core *core, pa_iochannel *io, pa_module *m) {
     char cname[256];
     pa_cli *c;
-    assert(io);
+    pa_assert(io);
 
-    c = pa_xmalloc(sizeof(pa_cli));
+    c = pa_xnew(pa_cli, 1);
     c->core = core;
-    c->line = pa_ioline_new(io);
-    assert(c->line);
+    pa_assert_se(c->line = pa_ioline_new(io));
 
     c->userdata = NULL;
     c->eof_callback = NULL;
 
     pa_iochannel_socket_peer_to_string(io, cname, sizeof(cname));
-    c->client = pa_client_new(core, __FILE__, cname);
-    assert(c->client);
+    pa_assert_se(c->client = pa_client_new(core, __FILE__, cname));
     c->client->kill = client_kill;
     c->client->userdata = c;
     c->client->owner = m;
@@ -88,13 +87,15 @@ pa_cli* pa_cli_new(pa_core *core, pa_iochannel *io, pa_module *m) {
     pa_ioline_set_callback(c->line, line_callback, c);
     pa_ioline_puts(c->line, "Welcome to PulseAudio! Use \"help\" for usage information.\n"PROMPT);
 
-    c->fail = c->kill_requested = c->defer_kill = 0;
+    c->fail = c->kill_requested = FALSE;
+    c->defer_kill = 0;
 
     return c;
 }
 
 void pa_cli_free(pa_cli *c) {
-    assert(c);
+    pa_assert(c);
+
     pa_ioline_close(c->line);
     pa_ioline_unref(c->line);
     pa_client_free(c->client);
@@ -103,12 +104,13 @@ void pa_cli_free(pa_cli *c) {
 
 static void client_kill(pa_client *client) {
     pa_cli *c;
-    assert(client && client->userdata);
-    c = client->userdata;
+
+    pa_assert(client);
+    pa_assert_se(c = client->userdata);
 
     pa_log_debug("CLI client killed.");
     if (c->defer_kill)
-        c->kill_requested = 1;
+        c->kill_requested = TRUE;
     else {
         if (c->eof_callback)
             c->eof_callback(c, c->userdata);
@@ -119,7 +121,9 @@ static void line_callback(pa_ioline *line, const char *s, void *userdata) {
     pa_strbuf *buf;
     pa_cli *c = userdata;
     char *p;
-    assert(line && c);
+
+    pa_assert(line);
+    pa_assert(c);
 
     if (!s) {
         pa_log_debug("CLI got EOF from user.");
@@ -129,8 +133,7 @@ static void line_callback(pa_ioline *line, const char *s, void *userdata) {
         return;
     }
 
-    buf = pa_strbuf_new();
-    assert(buf);
+    pa_assert_se(buf = pa_strbuf_new());
     c->defer_kill++;
     pa_cli_command_execute_line(c->core, s, buf, &c->fail);
     c->defer_kill--;
@@ -145,7 +148,8 @@ static void line_callback(pa_ioline *line, const char *s, void *userdata) {
 }
 
 void pa_cli_set_eof_callback(pa_cli *c, void (*cb)(pa_cli*c, void *userdata), void *userdata) {
-    assert(c);
+    pa_assert(c);
+
     c->eof_callback = cb;
     c->userdata = userdata;
 }
