@@ -1,7 +1,7 @@
 #ifndef foonativecommonhfoo
 #define foonativecommonhfoo
 
-/* $Id: native-common.h 1426 2007-02-13 15:35:19Z ossman $ */
+/* $Id: native-common.h 2064 2007-11-21 01:20:16Z lennart $ */
 
 /***
   This file is part of PulseAudio.
@@ -36,10 +36,10 @@ enum {
     PA_COMMAND_TIMEOUT, /* pseudo command */
     PA_COMMAND_REPLY,
 
-    /* Commands from client to server */
-    PA_COMMAND_CREATE_PLAYBACK_STREAM,
+    /* CLIENT->SERVER */
+    PA_COMMAND_CREATE_PLAYBACK_STREAM,        /* Payload changed in v9, v12 (0.9.0, 0.9.8) */
     PA_COMMAND_DELETE_PLAYBACK_STREAM,
-    PA_COMMAND_CREATE_RECORD_STREAM,
+    PA_COMMAND_CREATE_RECORD_STREAM,          /* Payload changed in v9, v12 (0.9.0, 0.9.8) */
     PA_COMMAND_DELETE_RECORD_STREAM,
     PA_COMMAND_EXIT,
     PA_COMMAND_AUTH,
@@ -64,8 +64,8 @@ enum {
     PA_COMMAND_GET_MODULE_INFO_LIST,
     PA_COMMAND_GET_CLIENT_INFO,
     PA_COMMAND_GET_CLIENT_INFO_LIST,
-    PA_COMMAND_GET_SINK_INPUT_INFO,
-    PA_COMMAND_GET_SINK_INPUT_INFO_LIST,
+    PA_COMMAND_GET_SINK_INPUT_INFO,          /* Payload changed in v11 (0.9.7) */
+    PA_COMMAND_GET_SINK_INPUT_INFO_LIST,     /* Payload changed in v11 (0.9.7) */
     PA_COMMAND_GET_SOURCE_OUTPUT_INFO,
     PA_COMMAND_GET_SOURCE_OUTPUT_INFO_LIST,
     PA_COMMAND_GET_SAMPLE_INFO,
@@ -92,18 +92,21 @@ enum {
     PA_COMMAND_KILL_CLIENT,
     PA_COMMAND_KILL_SINK_INPUT,
     PA_COMMAND_KILL_SOURCE_OUTPUT,
+
     PA_COMMAND_LOAD_MODULE,
     PA_COMMAND_UNLOAD_MODULE,
+
     PA_COMMAND_ADD_AUTOLOAD,
     PA_COMMAND_REMOVE_AUTOLOAD,
     PA_COMMAND_GET_AUTOLOAD_INFO,
     PA_COMMAND_GET_AUTOLOAD_INFO_LIST,
+
     PA_COMMAND_GET_RECORD_LATENCY,
     PA_COMMAND_CORK_RECORD_STREAM,
     PA_COMMAND_FLUSH_RECORD_STREAM,
     PA_COMMAND_PREBUF_PLAYBACK_STREAM,
 
-    /* Commands from server to client */
+    /* SERVER->CLIENT */
     PA_COMMAND_REQUEST,
     PA_COMMAND_OVERFLOW,
     PA_COMMAND_UNDERFLOW,
@@ -112,9 +115,30 @@ enum {
     PA_COMMAND_SUBSCRIBE_EVENT,
 
     /* A few more client->server commands */
+
+    /* Supported since protocol v10 (0.9.5) */
     PA_COMMAND_MOVE_SINK_INPUT,
     PA_COMMAND_MOVE_SOURCE_OUTPUT,
 
+    /* Supported since protocol v11 (0.9.7) */
+    PA_COMMAND_SET_SINK_INPUT_MUTE,
+
+    PA_COMMAND_SUSPEND_SINK,
+    PA_COMMAND_SUSPEND_SOURCE,
+
+    /* Supported since protocol v13 (0.9.8) */
+    PA_COMMAND_SET_PLAYBACK_STREAM_BUFFER_ATTR,
+    PA_COMMAND_SET_RECORD_STREAM_BUFFER_ATTR,
+
+    PA_COMMAND_UPDATE_PLAYBACK_STREAM_SAMPLE_RATE,
+    PA_COMMAND_UPDATE_RECORD_STREAM_SAMPLE_RATE,
+
+    /* SERVER->CLIENT */
+    PA_COMMAND_PLAYBACK_STREAM_SUSPENDED,
+    PA_COMMAND_RECORD_STREAM_SUSPENDED,
+    PA_COMMAND_PLAYBACK_STREAM_MOVED,
+    PA_COMMAND_RECORD_STREAM_MOVED,
+
     PA_COMMAND_MAX
 };
 
