@@ -1,7 +1,7 @@
 #ifndef foopackethfoo
 #define foopackethfoo
 
-/* $Id: packet.h 1426 2007-02-13 15:35:19Z ossman $ */
+/* $Id: packet.h 1971 2007-10-28 19:13:50Z lennart $ */
 
 /***
   This file is part of PulseAudio.
@@ -27,9 +27,11 @@
 #include <sys/types.h>
 #include <inttypes.h>
 
+#include <pulsecore/refcnt.h>
+
 typedef struct pa_packet {
+    PA_REFCNT_DECLARE;
     enum { PA_PACKET_APPENDED, PA_PACKET_DYNAMIC } type;
-    unsigned ref;
     size_t length;
     uint8_t *data;
 } pa_packet;
