@@ -1,7 +1,7 @@
 #ifndef foosampleutilhfoo
 #define foosampleutilhfoo
 
-/* $Id: sample-util.h 1426 2007-02-13 15:35:19Z ossman $ */
+/* $Id: sample-util.h 2041 2007-11-09 17:11:45Z lennart $ */
 
 /***
   This file is part of PulseAudio.
@@ -39,20 +39,35 @@ typedef struct pa_mix_info {
     pa_memchunk chunk;
     pa_cvolume volume;
     void *userdata;
+
+    /* The following fields are used internally by pa_mix(), should
+     * not be initialised by the caller of pa_mix(). */
+    void *ptr;
+    union {
+        int32_t i;
+        float f;
+    } linear[PA_CHANNELS_MAX];
 } pa_mix_info;
 
 size_t pa_mix(
-    const pa_mix_info channels[],
+    pa_mix_info channels[],
     unsigned nchannels,
     void *data,
     size_t length,
     const pa_sample_spec *spec,
     const pa_cvolume *volume,
-    int mute);
+    pa_bool_t mute);
 
 void pa_volume_memchunk(
     pa_memchunk*c,
     const pa_sample_spec *spec,
     const pa_cvolume *volume);
 
+size_t pa_frame_align(size_t l, const pa_sample_spec *ss) PA_GCC_PURE;
+
+int pa_frame_aligned(size_t l, const pa_sample_spec *ss) PA_GCC_PURE;
+
+void pa_interleave(const void *src[], unsigned channels, void *dst, size_t ss, unsigned n);
+void pa_deinterleave(const void *src, void *dst[], unsigned channels, size_t ss, unsigned n);
+
 #endif
