@@ -1,4 +1,8 @@
-/* $Id: hook-list-test.c 1418 2007-01-04 13:43:45Z ossman $ */
+/* $Id: hook-list-test.c 1971 2007-10-28 19:13:50Z lennart $ */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <pulsecore/hook-list.h>
 #include <pulsecore/log.h>
